/** Diese Klasse implementiert die Schlange fr z. B. einen Snaking Klon.
Die Schlange kann mit den Methoden moveSnake und resizeSnake bewegt und
vergrert werden. */

import java.util.*;

public class Snake
{
	// Variablen
	public int x_speed;						// x - Richtung der Schlange
	public int y_speed;						// y - Richtung der Schlange
	private int elements_counter;			// actuelle Anzahl der Elemente der Schlange
	private SnakeElement [] snake_elements;	// Array mit den Elementen der Schlange
	private Level level;					// Refferenz auf das Level

	/** Construktor, initialisiert die Variablen */
	public Snake ()
	{
		// Bewegungsrichtung am Anfang von links nach rechts
		x_speed = 1;
		y_speed = 0;

		// Anfngliche Zahl von Schlangenelementen
		elements_counter = 6;

		// Initialisierung des SchlangenElementeArrays
		snake_elements = new SnakeElement [elements_counter];

		// Initialisierung der Elemente
		initializeSnake (elements_counter);
	}

	/** Diese Funktion initialisiert die Elemente der Schlange
	mit Startwerten und zwar in Reihe 20, Spalte 1 */
	private void initializeSnake (int elements)
	{
		for (int i = 0; i < elements; i++)
		{
			snake_elements [i] =  new SnakeElement (20,  1);
		}
	}

	/** Mit dieser Funktion kann die Objectreferenz auf das Level
	gesetzt und verndert werden */
	public void setLevel (Level level)
	{
		this.level = level;
	}

	/** Diese Funktion bewegt die Schlange. Dazu wird das erste Element in
	Bewegungsrichtung verschoben. Alle anderen Elemente werden nachgezogen.
	Am Ende bermittelt die Funktion die Bereiche (Kopf und Ende der Schlange)
	wo etwas gendert wurde an das Level, wo das Levelarray dem entsprechend
	verndert wird */
	public void moveSnake ()
	{
		// Diese Variablen speichern den Arrayindex des neuen Kopfes der Schlange
		int head_i;
		int head_j;
		// Diese Variablen dienen der Zwischenspeicherung der Indexwerte
		int itemp;
		int jtemp;

		// Erstes Element der Schlange
		SnakeElement first =  snake_elements [0];

		// Zwischenspeichern der aktuellen Werte
		int old_i = first.getI();
		int old_j = first.getJ();

		// Bewegen des Kopfes der Schlange in xy_speed Richtung
		if (x_speed == 1) // Schlange bewegt sich nach links
		{
			// Wird also eine Spalte nach rechts verschoben
			int new_j = first.getJ () + 1;
			// Neue j - Position wird in SnakeElement gespeichert
			first.setJ (new_j);

			// Neue Position des Kopfes wird gespeichert
			head_j = new_j;
			head_i = old_i;
		}
		else if (x_speed == -1) // Schlange bewegt sich nach links
		{
			// Wird also eine Spalte nach links verschoben
			int new_j = first.getJ () - 1;
			// Neue j - Position wird in SnakeElement gespeichert
			first.setJ (new_j);

			// Neue Position des Kopfes wird gespeichert
			head_j = new_j;
			head_i = old_i;
		}
		else if (y_speed == 1) // Schlange bewegt sich nach unten
		{
			// Wird also eine Reihe nach unten verschoben
			int new_i = first.getI () + 1;
			// Neue i - Position wird in SnakeElement gespeichert
			first.setI (new_i);

			// Neue Position des Kopfes wird gespeichert
			head_j = old_j;
			head_i = new_i;
		}
		else // y_speed = -1, Schlange bewegt sich nach oben
		{
			// Wird also eine Reihe nach oben verschoben
			int new_i = first.getI () - 1;
			// Neue i - Position wird in SnakeElement gespeichert
			first.setI (new_i);

			// Neue Position des Kopfes wird gespeichert
			head_j = old_j;
			head_i = new_i;
		}

		/** Mit Hilfe dieser Schleife werden die Elemente der Schlange hinter
		dem Kopf hergezogen. Dazu werden die i und j Indexe eines Elements in
		dem Element dahinter gespeichert. So werden die Informationen immer an
		das nachfolgende Glied weitergegeben */
		for (int i = 1; i < snake_elements.length; i++)
		{
			SnakeElement actual = snake_elements[i];

			itemp = actual.getI ();
			jtemp = actual.getJ ();
			actual.setI (old_i);
			actual.setJ (old_j);
			old_i = itemp;
			old_j = jtemp;
		}

		/** Dieser Aufruf verndert nun das Levelarray an der Kopf und der
		Endstelle der Schlange. Auerdem kann diese Methode effizient testen,
		ob eine Kollision mit der Schlange selbst oder einem anderen Object
		(Wand, Powerup, ...) erfolgt ist */
		level.changeLevelArray (head_i, head_j, old_i, old_j);
	}

	/** Mit dieser Funktion wchst die Schlange. Dazu wird das momentane
	SnakeElements Array in ein Temp Array kopiert, das zwei Elements mehr
	fassen kann als das originale Array. Diesem neuen Array werden zwei neue
	Elemente in der Mitte der Schlange hinzugefgt. Die Schlange wchst so
	zunchst nicht, erst wenn die Elemente in der Mitte verndert werden,
	wchst die Schlange */
	public void resizeSnake ()
	{
		// Speichern des halben Wertes des Element Counters
		int elements_counter_halbe = elements_counter / 2;

		// Erzeugen eines temporren Arrays
		SnakeElement [] temp_array = new SnakeElement [elements_counter + 2];

		// Kopieren der ersten Hlfte der Element in temp Array
		for (int i = 0; i < (elements_counter_halbe); i++)
		{
			temp_array [i] = snake_elements [i];
		}

		// Hinzufgen zweier neuer Elemente
		temp_array [elements_counter_halbe] =
			new SnakeElement (snake_elements[elements_counter_halbe].getI(),
						  	  snake_elements[elements_counter_halbe].getJ());

		temp_array [elements_counter_halbe + 1] =
			new SnakeElement (snake_elements[elements_counter_halbe + 1].getI(),
						  	  snake_elements[elements_counter_halbe + 1].getJ());

		// Kopieren der zweiten Hlfte der Elemente
		for (int j = (elements_counter_halbe) + 2; j < temp_array.length; j++)
		{
			temp_array [j] = snake_elements [j - 2];
		}

		// Erhhen des Elements Counters
		elements_counter += 2;

		// temp_array wird neues snake_elements array
		snake_elements = temp_array;
	}
}









